unit GdiplusSub;

interface

uses Windows, ActiveX;

type
  TGpStatus = (
    gpstOk,
    gpstGenericError,
    gpstInvalidParameter,
    gpstOutOfMemory,
    gpstObjectBusy,
    gpstInsufficientBuffer,
    gpstNotImplemented,
    gpstWin32Error,
    gpstWrongState,
    gpstAborted,
    gpstFileNotFound,
    gpstValueOverflow,
    gpstAccessDenied,
    gpstUnknownImageFormat,
    gpstFontFamilyNotFound,
    gpstFontStyleNotFound,
    gpstNotTrueTypeFont,
    gpstUnsupportedGdiplusVersion,
    gpstGdiplusNotInitialized,
    gpstPropertyNotFound,
    gpstPropertyNotSupported
  );

  DebugEventLevel = (DebugEventLevelFatal, DebugEventLevelWarning);

  DebugEventProc = procedure(Level: DebugEventLevel; Message: PChar); stdcall;
  NotificationHookProc = function(out Token: ULONG): TGpStatus; stdcall;
  NotificationUnhookProc = procedure(Token: ULONG); stdcall;

  TGdiplusStartupInput = record
    GdiplusVersion: Cardinal; // UINT32
    DebugEventCallback: DebugEventProc;
    SuppressBackgroundThread: BOOL;
    SuppressExternalCodecs : BOOL;
  end;

  TGdiplusStartupOutput = record
    NotificationHook: NotificationHookProc;
    NotificationUnhook: NotificationUnhookProc;
  end;
  PGdiplusStartupOutput = ^TGdiPlusStartupOutput;

  TGpRect = record
    X: Integer;
    Y: Integer;
    Width: Integer;
    Height: Integer;
  end;

  TBitmapData = record
    Width: UINT;
    Height: UINT;
    Stride: Integer;
    PixelFormat: Integer;
    Scan0: Pointer;
    Reserved: UINT;
  end;

  PPropertyItem = ^TPropertyItem;
  TPropertyItem = record
    Id: PROPID;
    Length: ULONG;
    Type_: Word;
    Value: Pointer;
  end;

const
  ImageLockModeRead = 1;
  ImageLockModeWrite = 2;
  ImageLockModeUserInputBuf = 4;

  PixelFormatGDI = $00020000;
  PixelFormat24bppRGB = (8 or (24 shl 8) or PixelFormatGDI);

  ImageFormatGIF: TGUID = '{B96B3CB0-0728-11D3-9D7B-0000F81EF32E}';
  ImageFormatPNG: TGUID = '{B96B3CAF-0728-11D3-9D7B-0000F81EF32E}';

var
  GdiplusStartup: function (out Token: ULONG;
      const Input: TGdiplusStartupInput; Output: PGdiplusStartupOutput):
      TGpStatus; stdcall;
  GdiplusShutdown: procedure (Token: ULONG); stdcall;

  GdipDisposeImage: function (Image: Pointer): TGpStatus; stdcall;
  GdipGetImageWidth:function (Image: Pointer; var Width: UINT):
      TGpStatus; stdcall;
  GdipGetImageHeight: function (Image: Pointer; var Height: UINT):
      TGpStatus; stdcall;
  GdipGetImageRawFormat: function (Image: Pointer; var Format: TGUID):
      TGpStatus; stdcall;
  GdipGetImagePixelFormat: function (Image: Pointer; var Format: Integer):
      TGpStatus; stdcall;
  GdipGetPropertyCount: function (Image: Pointer; var NumOfProperty: UINT):
      TGpStatus; stdcall;
  GdipGetPropertyIdList: function (Image: Pointer; NumOfProperty: UINT;
      var list: PROPID): TGpStatus; stdcall;
  GdipGetPropertyItemSize: function (Image: Pointer; PropId: PROPID;
      var Size: UINT): TGpStatus; stdcall;
  GdipGetPropertyItem: function (Image: Pointer; PropId: PROPID;
      PropSize: UINT; var Buffer: TPropertyItem): TGpStatus; stdcall;

  GdipCreateBitmapFromStream: function (Stream: IStream; var Bitmap: Pointer):
      TGpStatus; stdcall;
  GdipCreateBitmapFromFile: function (const Filename: PWChar;
      var Bitmap: pointer): TGpStatus; stdcall;

  GdipBitmapLockBits: function (Bitmap: Pointer; const Rect: TGpRect;
      Flags: UINT; Format: Integer; var LockedBitmapData: TBitmapData):
      TGpStatus; stdcall;
  GdipBitmapUnlockBits: function (Bitmap: Pointer;
      const LockedBitmapData: TBitmapData): TGpStatus; stdcall;

implementation

var
  Handle: THandle;
begin
  Handle := LoadLibrary('gdiplus.dll');
  if Handle = 0 then
    Exit;
  GdiplusStartup := GetProcAddress(Handle, 'GdiplusStartup');
  GdiplusShutdown := GetProcAddress(Handle, 'GdiplusShutdown');

  GdipDisposeImage := GetProcAddress(Handle, 'GdipDisposeImage');
  GdipGetImageWidth := GetProcAddress(Handle, 'GdipGetImageWidth');
  GdipGetImageHeight := GetProcAddress(Handle, 'GdipGetImageHeight');
  GdipGetImageRawFormat := GetProcAddress(Handle, 'GdipGetImageRawFormat');
  GdipGetImagePixelFormat := GetProcAddress(Handle, 'GdipGetImagePixelFormat');
  GdipGetPropertyCount := GetProcAddress(Handle, 'GdipGetPropertyCount');
  GdipGetPropertyIdList := GetProcAddress(Handle, 'GdipGetPropertyIdList');
  GdipGetPropertyItemSize := GetProcAddress(Handle, 'GdipGetPropertyItemSize');
  GdipGetPropertyItem := GetProcAddress(Handle, 'GdipGetPropertyItem');

  GdipCreateBitmapFromStream :=
      GetProcAddress(Handle, 'GdipCreateBitmapFromStream');
  GdipCreateBitmapFromFile :=
      GetProcAddress(Handle, 'GdipCreateBitmapFromFile');
  GdipBitmapLockBits := GetProcAddress(Handle, 'GdipBitmapLockBits');
  GdipBitmapUnlockBits := GetProcAddress(Handle, 'GdipBitmapUnlockBits');
end.
